# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-22 14:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('thorbanks', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.FloatField()),
                ('is_paid', models.BooleanField(default=False)),
                ('transaction', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='thorbanks.Transaction')),
            ],
        ),
    ]
