from __future__ import unicode_literals

import os
import sys


BASE_DIR = os.path.dirname(__file__)

sys.path.append(os.path.join(BASE_DIR, '..'))
sys.path.append(os.path.join(BASE_DIR, '..', 'example'))
