<html>
    <head>
        <title>Banklink request</title>
        <meta http-equiv="Content-Type" value="text/html; charset=utf-8" />
    </head>
    <body>
        {{ form.redirect_html|safe }}
    </body>
</html>
