<html>
    <head>
        <title>Test Swedbank banklink response</title>
        <meta http-equiv="Content-Type" value="text/html; charset=utf-8" />
    </head>
    <body>
        {% if signature_valid %}
            <h2>swedbank signature valid</h2>
        {% else %}
            <h2>swedbank signature invalid</h2>
        {% endif %}
        <table>
            {% for value, key in data %}
            <tr>
                <td>{{ key }}</td>
                <td>{{ value }}</td>
            </tr>
            {% endfor %}
        </table>
    </body>
</html>
